#!/usr/bin/env python3
"""WRD (Word) - Python Package

A powerful workflow automation tool for developers, inspired by Claude Code workflow.

Features:
- Project management
- Documentation automation
- AI tools integration
- Workflow optimization
- Interactive shell
- Project templates
"""

import json
import logging
import os
import signal
import subprocess
import sys
import yaml
from datetime import datetime
from pathlib import Path
from typing import Dict, List, Optional, Any, TypeVar, Union, Tuple

import typer
from rich.console import Console
from rich.logging import RichHandler
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn
from rich.table import Table
from rich.prompt import Confirm, Prompt

from . import __version__
from .cli import WRDShell
from .template_manager import get_template_manager

# Configure logging
logging.basicConfig(
    level=logging.INFO,
    format="%(message)s",
    datefmt="[%X]",
    handlers=[
        RichHandler(rich_tracebacks=True, markup=True, show_time=False, show_path=False),
        logging.FileHandler(Path.home() / ".wrd" / "wrd.log")
    ]
)
logger = logging.getLogger("wrd")
console = Console()

# Type variable for generic function return type
T = TypeVar("T")

# Initialize Typer app
app = typer.Typer(
    name="wrd",
    help="WRD (WRonai Development) - A powerful workflow tool for developers",
    add_completion=False,
    no_args_is_help=True,
    rich_markup_mode="rich"
)


class WRDConfig:
    """WRD Configuration Manager"""

    def __init__(self):
        self.home_dir = Path.home()
        self.wrd_dir = self.home_dir / ".wrd"
        self.config_file = self.wrd_dir / "config.yaml"
        self.projects_dir = Path(os.getenv("WRD_PROJECTS_DIR", self.home_dir / "projects"))
        self.templates_dir = self.wrd_dir / "templates"
        self.user_templates_dir = self.wrd_dir / "user_templates"
        self.cache_dir = self.wrd_dir / "cache"
        
        # Ensure all required directories exist
        self.ensure_directories()
        self.config = self.load_config()

    def ensure_directories(self) -> None:
        """Ensure all required directories exist."""
        directories = [
            self.wrd_dir,
            self.projects_dir,
            self.templates_dir,
            self.user_templates_dir,
            self.cache_dir
        ]
        for directory in directories:
            directory.mkdir(parents=True, exist_ok=True)

    def load_config(self) -> Dict[str, Any]:
        """Load configuration from YAML file."""
        if not self.config_file.exists():
            return self.get_default_config()

        try:
            with open(self.config_file, 'r') as f:
                config = yaml.safe_load(f) or {}
                return {**self.get_default_config(), **config}
        except Exception as e:
            logger.error(f"Error loading configuration: {e}")
            return self.get_default_config()

    def save_config(self) -> None:
        """Save configuration to YAML file."""
        try:
            with open(self.config_file, 'w') as f:
                yaml.dump(self.config, f, default_flow_style=False)
        except Exception as e:
            logger.error(f"Error saving configuration: {e}")

    def get_default_config(self) -> Dict[str, Any]:
        """Return default configuration."""
        return {
            'version': '1.0.0',
            'projects_dir': str(self.projects_dir),
            'editor': os.getenv('EDITOR', 'code'),
            'default_language': 'python',
            'default_template': 'python',
            'templates': {},
            'ai_tools': {
                'claude_code': {
                    'enabled': True,
                    'api_key': os.getenv('CLAUDE_API_KEY', '')
                }
            },
            'workflows': {
                'documentation_auto': True,
                'commit_auto_describe': True
            },
            'recent_projects': [],
            'settings': {
                'auto_update': True,
                'check_updates': True,
                'notifications': True
            }
        }


class WRDProject:
    """Klasa reprezentująca projekt WRD"""

    def __init__(self, name: str, project_type: str = 'python'):
        self.name = name
        self.project_type = project_type
        self.config = WRDConfig()
        self.project_dir = self.config.projects_dir / name
        self.claude_md_file = self.project_dir / 'CLAUDE.md'

    def create(self, description: str = ""):
        """Tworzenie nowego projektu"""
        if self.project_dir.exists():
            logger.warning(f"Projekt {self.name} już istnieje")
            return False

        # Tworzenie struktury katalogów
        dirs = ['src', 'tests', 'docs', 'scripts', 'config']
        for dir_name in dirs:
            (self.project_dir / dir_name).mkdir(parents=True)

        # Tworzenie plików bazowych
        self._create_readme(description)
        self._create_claude_md(description)
        self._create_requirements_txt()
        self._create_gitignore()

        # Inicjalizacja git
        subprocess.run(['git', 'init'], cwd=self.project_dir, capture_output=True)

        logger.info(f"Projekt {self.name} został utworzony w {self.project_dir}")
        return True

    def _create_readme(self, description: str):
        """Tworzenie README.md"""
        readme_content = f"""# {self.name}

{description}

## Opis projektu
Projekt typu: {self.project_type}
Utworzony: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M')}

## Instalacja
```bash
cd {self.name}
python -m venv venv
source venv/bin/activate  # Linux/Mac
pip install -r requirements.txt
```

## Użycie
```bash
python src/main.py
```

## Rozwój
Ten projekt został utworzony z użyciem WRD (Word) - narzędzia do zarządzania projektami Claude Code.

## Dokumentacja
Szczegółowa dokumentacja znajduje się w pliku CLAUDE.md
"""

        with open(self.project_dir / 'README.md', 'w') as f:
            f.write(readme_content)

    def _create_claude_md(self, description: str):
        """Tworzenie CLAUDE.md dla Claude Code"""
        claude_content = f"""# Claude Code Project: {self.name}

## Przegląd projektu
- **Nazwa**: {self.name}
- **Typ**: {self.project_type}
- **Opis**: {description}
- **Utworzony**: {datetime.datetime.now().strftime('%Y-%m-%d %H:%M')}
- **Narzędzie**: WRD (Word) v1.0

## Środowisko deweloperskie
- **OS**: Fedora Linux
- **Python**: {sys.version.split()[0]}
- **Virtual Environment**: venv/
- **Narzędzia**: Claude Code, WRD

## Struktura projektu
```
{self.name}/
├── src/           # Kod źródłowy
├── tests/         # Testy
├── docs/          # Dokumentacja
├── scripts/       # Skrypty pomocnicze
├── config/        # Konfiguracja
├── venv/          # Środowisko wirtualne
├── README.md      # Podstawowa dokumentacja
├── CLAUDE.md      # Ten plik - dokumentacja dla Claude Code
├── requirements.txt # Zależności Python
└── .gitignore     # Git ignore
```

## Workflow z Claude Code
1. **Planowanie** (Gemini 2.5 Pro/Claude.ai)
   - Architektura rozwiązania
   - Specyfikacja funkcjonalności
   - Planowanie iteracji

2. **Implementacja** (Claude Code)
   - Kodowanie w 5-godzinnych sesjach
   - Automatyczne commity z opisami
   - Iteracyjny rozwój

3. **Dokumentacja** (Automatyczna)
   - WRD automatycznie dokumentuje proces
   - Śledzenie błędów i rozwiązań
   - Historia zmian

## Postęp prac
### {datetime.datetime.now().strftime('%Y-%m-%d')}
- ✅ Inicjalizacja projektu
- ✅ Struktura katalogów
- ⏳ Implementacja podstawowej funkcjonalności

## Błędy i rozwiązania
<!-- Automatycznie aktualizowane przez WRD -->

## Notatki techniczne
<!-- Miejsce na notatki specyficzne dla Claude Code -->

## Optymalizacje wydajności
<!-- Dokumentacja optymalizacji -->

## Deployment
<!-- Instrukcje wdrożenia -->

---
*Dokumentacja generowana przez WRD (Word) - narzędzie workflow dla Claude Code*
"""

        with open(self.claude_md_file, 'w') as f:
            f.write(claude_content)

    def _create_requirements_txt(self):
        """Tworzenie requirements.txt"""
        requirements = [
            "requests>=2.28.0",
            "click>=8.0.0",
            "rich>=12.0.0",
            "pydantic>=1.9.0",
            "python-dotenv>=0.19.0"
        ]

        if self.project_type == 'fastapi':
            requirements.extend([
                "fastapi>=0.95.0",
                "uvicorn>=0.20.0"
            ])
        elif self.project_type == 'data':
            requirements.extend([
                "pandas>=1.5.0",
                "numpy>=1.24.0",
                "matplotlib>=3.6.0"
            ])

        with open(self.project_dir / 'requirements.txt', 'w') as f:
            f.write('\n'.join(requirements))

    def _create_gitignore(self):
        """Tworzenie .gitignore"""
        gitignore_content = """# WRD Generated .gitignore

# Python
__pycache__/
*.py[cod]
*$py.class
*.so
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg

# Virtual Environment
venv/
env/
ENV/

# IDE
.vscode/

class WRDManager:
    """Main class for managing WRD projects."""

    def __init__(self, config: 'WRDConfig'):
        """Initialize WRDManager with config."""
        self.config = config
        self.projects: Dict[str, WRDProject] = {}
        self.load_projects()
    
    def load_projects(self) -> None:
        """Load all projects from the projects directory."""
        self.projects = {}
        if not self.config.projects_dir.exists():
            return
            
        for project_dir in self.config.projects_dir.glob('*'):
            if project_dir.is_dir() and not project_dir.name.startswith('.'):
                try:
                    project = WRDProject(project_dir, self.config)
                    self.projects[project.name] = project
                except Exception as e:
                    logger.warning(f"Error loading project {project_dir.name}: {e}")
    
    def create_project(
        self, 
        name: str, 
        project_type: str = 'python', 
        description: str = '',
        template: Optional[str] = None,
        **kwargs
    ) -> 'WRDProject':
        """Create a new project.
        
        Args:
            name: Name of the project
            project_type: Type of project (e.g., 'python', 'web', 'data')
            description: Project description
            template: Optional template to use for project creation
            **kwargs: Additional project metadata
            
        Returns:
            WRDProject: The created project
            
        Raises:
            ValueError: If project already exists or creation fails
        """
        project_path = self.config.projects_dir / name
        
        if project_path.exists():
            raise ValueError(f"Project '{name}' already exists")
        
        try:
            # Create project directory structure
            project_path.mkdir(parents=True, exist_ok=True)
            
            # Initialize project with metadata
            project = WRDProject(project_path, self.config)
            project._metadata.update({
                'name': name,
                'type': project_type,
                'description': description,
                'created_at': datetime.now().isoformat(),
                'status': 'active',
                'ai_tools': [],
                'workflows': {},
                **kwargs
            })
            
            # Apply template if specified
            if template:
                self._apply_template(project, template)
            
            # Initialize Git repository
            self._init_git_repo(project_path)
            
            # Save metadata and add to projects
            project.save_metadata()
            self.projects[name] = project
            
            # Update recent projects
            self._update_recent_projects(name)
            
            return project
            
        except Exception as e:
            # Clean up on failure
            if project_path.exists():
                import shutil
                shutil.rmtree(project_path, ignore_errors=True)
            raise ValueError(f"Failed to create project: {e}")
    
    def _apply_template(self, project: 'WRDProject', template_name: str) -> None:
        """Apply a template to the project.
        
        Args:
            project: The project to apply the template to
            template_name: Name of the template to apply
            
        Raises:
            ValueError: If template is not found or application fails
        """
        try:
            template_manager = get_template_manager()
            context = {
                'project_name': project.name,
                'project_path': str(project.path),
                'project_type': project._metadata.get('type', 'python'),
                'description': project._metadata.get('description', ''),
                'author': os.getenv('USER', 'Your Name'),
                'email': '',
                'year': datetime.now().year,
                'version': '0.1.0',
            }
            
            success = template_manager.create_project(
                template_name=template_name,
                project_path=project.path,
                context=context,
                overwrite=False
            )
            
            if not success:
                raise ValueError(f"Failed to apply template: {template_name}")
                
        except Exception as e:
            logger.error(f"Error applying template: {e}")
            if os.getenv("WRD_DEBUG"):
                import traceback
                logger.debug(traceback.format_exc())
            raise
    
    def _init_git_repo(self, path: Path) -> None:
        """Initialize a Git repository for the project."""
        try:
            # Initialize repository
            run_command(['git', 'init'], cwd=path)
            
            # Configure basic Git settings
            run_command(['git', 'config', 'pull.rebase', 'false'], cwd=path)
            
            # Create basic .gitignore if it doesn't exist
            gitignore = path / '.gitignore'
            if not gitignore.exists():
                gitignore_content = (
                    "# Python\n"
                    "__pycache__/\n"
                    "*.py[cod]\n"
                    "*$py.class\n"
                    "*.so\n"
                    ".Python\n"
                    "build/\n"
                    "develop-eggs/\n"
                    "dist/\n"
                    "downloads/\n"
                    "eggs/\n"
                    ".eggs/\n"
                    "lib\n"
                    "lib64\n"
                    "parts/\n"
                    "sdist/\n"
                    "var/\n"
                    "wheels/\n"
                    "*.egg-info/\n"
                    ".env\n"
                    ".venv\n"
                    "venv/\n"
                    "ENV/\n"
                    ".mypy_cache/\n"
                    ".pytest_cache/\n"
                    ".coverage\n"
                    "htmlcov/\n"
                    ".DS_Store\n"
                    "Thumbs.db\n"
                )
                with open(gitignore, 'w') as f:
                    f.write(gitignore_content)
            
            # Make initial commit
            run_command(['git', 'add', '.'], cwd=path)
            run_command(
                ['git', 'commit', '-m', 'Initial commit'],
                cwd=path
            )
            
        except Exception as e:
            logger.warning(f"Git repository initialization failed: {e}")
    
    def _update_recent_projects(self, project_name: str) -> None:
        """Update the list of recently used projects."""
        recent = self.config.config.get('recent_projects', [])
        if project_name in recent:
            recent.remove(project_name)
        recent.insert(0, project_name)
        self.config.config['recent_projects'] = recent[:10]  # Keep only 10 most recent
        self.config.save_config()
    
    def list_projects(self, status: Optional[str] = None) -> List['WRDProject']:
        """List all projects with optional status filter."""
        if status:
            return [p for p in self.projects.values() if p.get_status() == status]
        return list(self.projects.values())
    
    def get_project(self, name: str) -> Optional['WRDProject']:
        """Get a project by name."""
        return self.projects.get(name)
    
    def delete_project(self, name: str, force: bool = False) -> bool:
        """Delete a project.

        Args:
            name (str): Name of the project to delete
            force (bool, optional): If True, don't ask for confirmation. Defaults to False.

        Returns:
            bool: True if project was deleted, False otherwise
        """
        if name not in self.projects:
            logger.error(f"Project '{name}' not found")
            return False
        
        project = self.projects[name]
        
        if not force:
            confirm = click.confirm(
                f"Are you sure you want to delete project '{name}'? This cannot be undone.",
                default=False
            )
            if not confirm:
                return False
        
        try:
            if project.path.exists():
                import shutil
                shutil.rmtree(project.path, ignore_errors=True)
            
            # Remove from projects list and recent projects
            del self.projects[name]
            self._remove_from_recent_projects(name)
            
            logger.info(f"Project '{name}' deleted successfully")
            return True
            
        except Exception as e:
            logger.error(f"Failed to delete project '{name}': {e}")
            return False
    
    def _remove_from_recent_projects(self, project_name: str) -> None:
        """Remove a project from recent projects list."""
        if 'recent_projects' in self.config.config:
            recent = self.config.config['recent_projects']
            if project_name in recent:
                recent.remove(project_name)
                self.config.save_config()
    
    def auto_commit(self, message: Optional[str] = None) -> None:
        """Automatically commit changes in all projects."""
        for project in self.projects.values():
            if (project.path / '.git').exists():
                try:
                    # Check for changes
                    result = run_command(
                        ['git', 'status', '--porcelain'],
                        cwd=project.path,
                        capture_output=True
                    )
                    
                    if result.stdout.strip():
                        # There are changes to commit
                        if not message:
                            message = f"Auto-commit: {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
                        
                        run_command(['git', 'add', '.'], cwd=project.path)
                        run_command(
                            ['git', 'commit', '-m', message],
                            cwd=project.path
                        )
                        logger.info(f"Committed changes in {project.name}")
                        
                except Exception as e:
                    logger.error(f"Failed to auto-commit in {project.name}: {e}")
    
    def show_status(self) -> None:
        """Show status of all projects."""
        projects = self.list_projects()
        if not projects:
            console.print("[yellow]No projects found.[/]")
            return

        table = Table(title="WRD Projects")
        table.add_column("Name", style="cyan")
        table.add_column("Type")
        table.add_column("Status")
        table.add_column("Path")
        
        for project in sorted(projects, key=lambda p: p.name):
            project_dir = self.config.projects_dir / project.name
            
            # Get Git status if available
            git_status = ""
            if (project_dir / '.git').exists():
                try:
                    # Check if there are uncommitted changes
                    result = run_command(
                        ['git', 'status', '--porcelain'],
                        cwd=project_dir,
                        capture_output=True
                    )
                    if result.stdout.strip():
                        git_status = "[yellow]uncommitted[/]"
                    else:
                        git_status = "[green]clean[/]"
                except Exception:
                    git_status = "[red]error[/]"
            
            table.add_row(
                project.name,
                project._metadata.get('type', 'unknown'),
                git_status or "[dim]no git[/]",
                str(project.path)
            )
        
        console.print(table)


# Helper functions
def run_command(
    cmd: Union[str, List[str]], 
    cwd: Optional[Union[str, Path]] = None, 
    capture_output: bool = False,
    check: bool = True
) -> subprocess.CompletedProcess:
    """Run a shell command with proper error handling.
    
    Args:
        cmd: Command to run as string or list of arguments
        cwd: Working directory for the command
        capture_output: Whether to capture command output
        check: Whether to raise an exception on non-zero exit code
        
    Returns:
        CompletedProcess object with command results
    """
    try:
        if isinstance(cmd, str):
            # Use shell=True for string commands
            return subprocess.run(
                cmd,
                shell=True,
                cwd=str(cwd) if cwd else None,
                check=check,
                capture_output=capture_output,
                text=True,
                encoding='utf-8',
                errors='replace'
            )
        else:
            # Use shell=False for list of arguments (safer)
            return subprocess.run(
                [str(arg) for arg in cmd],
                shell=False,
                cwd=str(cwd) if cwd else None,
                check=check,
                capture_output=capture_output,
                text=True,
                encoding='utf-8',
                errors='replace'
            )
    except subprocess.CalledProcessError as e:
        logger.error(f"Command failed with exit code {e.returncode}: {e.cmd}")
        if e.stdout:
            logger.debug(f"Command output:\n{e.stdout}")
        if e.stderr:
            logger.error(f"Command error:\n{e.stderr}")
        raise
    except FileNotFoundError as e:
        logger.error(f"Command not found: {e.filename}")
        raise
    except Exception as e:
        logger.error(f"Unexpected error running command: {e}")
        raise


def handle_keyboard_interrupt() -> None:
    """Handle keyboard interrupt gracefully."""
    console.print("\n[red]Operation cancelled by user.[/]")
    sys.exit(1)


# Register signal handler for graceful exit
signal.signal(signal.SIGINT, lambda *_: handle_keyboard_interrupt())


# CLI Commands
@app.command()
def shell():
    """Start interactive WRD shell."""
    try:
        WRDShell().shell()
    except Exception as e:
        logger.error(f"Error in shell: {e}")
        raise typer.Exit(1)


@app.command()
def init(
    name: str = typer.Argument(..., help="Project name"),
    path: Path = typer.Argument(
        None,
        exists=False,
        file_okay=False,
        dir_okay=True,
        writable=True,
        resolve_path=True,
        help="Project path (default: current directory/name)",
    ),
    template: str = typer.Option(
        None, "--template", "-t", help="Template to use for the project"
    ),
    description: str = typer.Option(
        "", "--description", "-d", help="Project description"
    ),
    interactive: bool = typer.Option(
        True, "--interactive/--no-interactive", "-i/-I", help="Interactive mode"
    ),
):
    """Initialize a new project."""
    try:
        config = get_config()
        template_manager = get_template_manager()
        
        # Set default path if not provided
        if path is None:
            path = Path.cwd() / name
        
        # List available templates if none specified
        available_templates = template_manager.list_templates()
        if not template and interactive:
            if not available_templates:
                console.print("[yellow]No templates found. Using default Python template.[/]")
                template = "python"
            else:
                table = Table(title="Available Templates")
                table.add_column("Name", style="cyan")
                table.add_column("Description")
                
                for tpl_name in available_templates:
                    tpl = template_manager.get_template(tpl_name)
                    desc = tpl['config'].get('description', 'No description')
                    table.add_row(tpl_name, desc)
                
                console.print(table)
                template = console.input("\n[bold]Choose a template[/] (default: python): ") or "python"
        
        # Get project details interactively
        if interactive:
            name = console.input(f"Project name [[cyan]{name}[/]]: ") or name
            path = Path(console.input(f"Project path [[cyan]{path}[/]]: ") or path)
            description = console.input(f"Description [[cyan]{description or 'No description'}[/]]: ") or description or ""
        
        # Create project context
        context = {
            "project_name": name,
            "package_name": name.lower().replace("-", "_").replace(" ", "_"),
            "description": description,
            "author_name": os.getenv("USER", "Your Name"),
            "author_email": "",
            "license": "MIT",
            "year": datetime.now().year,
        }
        
        # Create project
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            transient=True,
        ) as progress:
            progress.add_task(description="Creating project...", total=None)
            success = template_manager.create_project(
                template_name=template,
                project_path=path,
                context=context,
                overwrite=False,
            )
        
        if success:
            console.print(f"\n[green]✓ Project created at {path}[/]")
            
            # Open in editor if configured
            editor = config.config.get("editor")
            if editor and interactive and typer.confirm("Open project in editor?"):
                run_command(f"{editor} {path}")
        else:
            console.print("[red]Failed to create project.[/]")
            raise typer.Exit(1)
            
    except Exception as e:
        logger.error(f"Error initializing project: {e}")
        raise typer.Exit(1)


@app.command()
def list_templates():
    """List all available project templates."""
    try:
        template_manager = get_template_manager()
        templates = template_manager.list_templates()
        
        if not templates:
            console.print("[yellow]No templates found.[/]")
            return
        
        table = Table(
            title="Available Templates",
            caption=f"Use 'wrd init --template <name>' to create a project with a template"
        )
        table.add_column("Name", style="cyan", no_wrap=True)
        table.add_column("Description")
        table.add_column("Type", no_wrap=True)
        
        # Sort templates by name
        for name in sorted(templates):
            template = template_manager.get_template(name)
            if not template:
                continue
                
            desc = template.get('config', {}).get('description', 'No description')
            
            # Determine template type
            template_path = str(template.get('path', ''))
            if "site-packages" in template_path:
                tpl_type = "Built-in"
            elif str(Path.home()) in template_path:
                tpl_type = "User"
            else:
                tpl_type = "System"
                
            table.add_row(name, desc, tpl_type)
        
        console.print(table)
        
    except Exception as e:
        logger.error(f"Error listing templates: {e}")
        if os.getenv("WRD_DEBUG"):
            import traceback
            logger.debug(traceback.format_exc())
        raise typer.Exit(1)


@app.command()
def config():
    """Configure WRD settings."""
    config = get_config()
    
    console.print("[bold]Current Configuration:[/]")
    console.print_json(json.dumps(config.config, indent=2, default=str))
    
    if typer.confirm("\nDo you want to edit the configuration?"):
        editor = os.getenv("EDITOR", "nano")
        os.system(f"{editor} {config.config_file}")


@app.command()
def version():
    """Show WRD version."""
    console.print(f"WRD version: [bold cyan]{__version__}[/]")

if __name__ == "__main__":
    app()


"""Unit tests for the TemplateManager class."""

import os
import shutil
import tempfile
from pathlib import Path
from unittest.mock import patch, MagicMock

import pytest
import yaml

from wrd.template_manager import TemplateManager


class TestTemplateManager:
    """Test cases for the TemplateManager class."""

    @pytest.fixture(autouse=True)
    def setup(self, tmp_path):
        """Set up test fixtures."""
        self.temp_dir = tmp_path
        self.templates_dir = self.temp_dir / "templates"
        self.templates_dir.mkdir()

        # Create a test template
        self.test_template = self.templates_dir / "python-basic"
        self.test_template.mkdir()

        # Create a project.yml for the test template
        self.template_config = {
            "name": "python-basic",
            "description": "Basic Python project template",
            "author": "WRD Team",
            "version": "1.0.0",
            "variables": [
                {"name": "project_name", "description": "Name of the project"},
                {"name": "author", "description": "Author name"},
            ],
            "files": [
                {
                    "source": "README.md",
                    "content": "# {{project_name}}\n\nCreated by {{author}}",
                },
                {
                    "source": "setup.py",
                    "content": "from setuptools import setup\n\nsetup(\n    name='{{project_name}}',\n    version='0.1.0',\n    author='{{author}}'\n)",
                },
                {"source": "src/{{project_name}}/__init__.py", "content": ""},
                {"source": "tests/__init__.py", "content": ""},
            ],
            "directories": ["src/{{project_name}}", "tests"],
        }

        with open(self.test_template / "project.yml", "w") as f:
            yaml.dump(self.template_config, f)

        # Create a test file in the template
        (self.test_template / "test_file.txt").write_text("Test content")

        # Patch pkg_resources to use our test templates directory
        self.patcher = patch('wrd.template_manager.pkg_resources.files')
        self.mock_files = self.patcher.start()
        self.mock_files.return_value = self.templates_dir

        yield

        # Cleanup
        self.patcher.stop()

    def test_discover_templates(self):
        """Test that templates are discovered correctly."""
        manager = TemplateManager()
        templates = manager.available_templates

        assert "python-basic" in templates
        assert templates["python-basic"]["config"]["name"] == "python-basic"
        assert templates["python-basic"]["path"] == self.test_template

    def test_get_template(self):
        """Test getting a template by name."""
        manager = TemplateManager()
        template = manager.get_template("python-basic")

        assert template is not None
        assert template["config"]["name"] == "python-basic"

        # Test non-existent template
        assert manager.get_template("nonexistent") is None

    def test_list_templates(self):
        """Test listing available templates."""
        manager = TemplateManager()
        templates = manager.list_templates()

        assert isinstance(templates, list)
        assert "python-basic" in templates

    def test_create_project(self):
        """Test creating a project from a template."""
        manager = TemplateManager()
        project_path = self.temp_dir / "output" / "test-project"
        project_path.parent.mkdir(parents=True, exist_ok=True)

        context = {"project_name": "test-project", "author": "Test User"}

        result = manager.create_project(
            template_name="python-basic",
            project_path=project_path,
            context=context,
            overwrite=False,
        )

        # Verify the project directory was created
        assert project_path.exists()
        assert (project_path / "README.md").exists()

        # Verify the template files were created
        assert (project_path / "src" / "test-project").exists()
        assert (project_path / "tests").exists()
        assert (project_path / "setup.py").exists()
        assert (project_path / "src" / "test-project" / "__init__.py").exists()
        assert (project_path / "tests" / "__init__.py").exists()

        # Verify the context was applied to templates
        readme = (project_path / "README.md").read_text()
        assert "test-project" in readme
        assert "Test User" in readme

    def test_create_project_invalid_template(self):
        """Test creating a project with an invalid template name."""
        manager = TemplateManager()
        project_path = self.temp_dir / "output" / "test-project"
        project_path.parent.mkdir(parents=True, exist_ok=True)

        with pytest.raises(ValueError, match="Template 'nonexistent' not found"):
            manager.create_project(
                template_name="nonexistent",
                project_path=project_path,
                context={"project_name": "test"},
            )

    def test_create_project_existing_output(self):
        """Test creating a project when output directory exists."""
        manager = TemplateManager()
        project_path = self.temp_dir / "output" / "test-project"
        project_path.parent.mkdir(parents=True, exist_ok=True)
        project_path.mkdir()  # Create the project directory to cause conflict

        # Test with overwrite=False (should raise FileExistsError)
        with pytest.raises(FileExistsError):
            manager.create_project(
                template_name="python-basic",
                project_path=project_path,
                context={"project_name": "test-project"},
                overwrite=False,
            )

        # Test with overwrite=True (should work)
        result = manager.create_project(
            template_name="python-basic",
            project_path=project_path,
            context={"project_name": "test-project"},
            overwrite=True,
        )
        assert result is True  # Should complete successfully with overwrite



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'WRD'
copyright = '2023, WRonai Team'
author = 'WRonai Team'
release = '0.1.0'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
    'sphinx.ext.intersphinx',
    'sphinx.ext.todo',
    'sphinx_copybutton',
    'myst_parser',
]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']
html_theme_options = {
    'navigation_depth': 4,
    'collapse_navigation': False,
    'sticky_navigation': True,
    'includehidden': True,
    'titles_only': False
}

# -- Extension configuration -------------------------------------------------

# Intersphinx configuration
intersphinx_mapping = {
    'python': ('https://docs.python.org/3', None),
    'typer': ('https://typer.tiangolo.com/', None),
}

# Autodoc configuration
autodoc_default_options = {
    'members': True,
    'member-order': 'bysource',
    'special-members': '__init__',
    'undoc-members': True,
    'exclude-members': '__weakref__',
}

# Napoleon settings
napoleon_google_docstring = True
napoleon_numpy_docstring = False
napoleon_include_init_with_doc = True
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = False
napoleon_use_admonition_for_notes = False
napoleon_use_admonition_for_references = False
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True
napoleon_preprocess_types = False
napoleon_type_aliases = None
napoleon_attr_annotations = True

# MyST settings
myst_enable_extensions = [
    'colon_fence',
    'deflist',
    'dollarmath',
    'fieldlist',
    'html_admonition',
    'html_image',
    'linkify',
    'replacements',
    'smartquotes',
    'strikethrough',
    'substitution',
    'tasklist',
]

# Copy button settings
copybutton_prompt_text = r'>>> |\.\.\. |\$ |In \[\d+\]: |\s*\.\.\.: |\s*\$ |\s*In \[\d+\]: ' 
copybutton_prompt_is_regexp = True

# -- Options for todo extension ----------------------------------------------
# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = True



"""Configuration file for pytest."""

import os
import sys
from pathlib import Path
from typing import Generator, Any, Dict

import pytest

# Add the src directory to the Python path
sys.path.insert(0, str(Path(__file__).parent.parent / "src"))


@pytest.fixture(scope="session", autouse=True)
def setup_test_environment() -> Generator[None, None, None]:
    """Set up the test environment."""
    # Set up any test environment variables here
    os.environ["WRD_DEBUG"] = "1"

    # Yield to run the tests
    yield

    # Clean up after tests if needed


@pytest.fixture
def tmp_project_dir(tmp_path):
    """Create a temporary project directory for testing."""
    project_dir = tmp_path / "test_project"
    project_dir.mkdir()
    return project_dir


@pytest.fixture
def template_manager():
    """Create a template manager instance for testing."""
    from wrd.template_manager import TemplateManager

    return TemplateManager()


@pytest.fixture
def default_template_data() -> Dict[str, Any]:
    """Return default template data for testing."""
    return {
        "project_name": "test_project",
        "author": "Test User",
        "email": "test@example.com",
        "description": "A test project",
        "license": "MIT",
        "version": "0.1.0",
    }



"""Template manager for WRD."""

import os
import shutil
from pathlib import Path
from typing import Dict, Any, Optional, List
import yaml
import jinja2
import importlib.resources as pkg_resources
from . import templates


class TemplateManager:
    """Manages project templates for different languages and project types."""

    def __init__(self):
        self.templates_dir = Path(pkg_resources.files(templates))
        self.available_templates = self._discover_templates()

    def _discover_templates(self) -> Dict[str, Dict[str, Any]]:
        """Discover all available templates in the templates directory."""
        templates = {}

        if not self.templates_dir.exists():
            return {}

        for template_dir in self.templates_dir.iterdir():
            if template_dir.is_dir() and (template_dir / 'project.yml').exists():
                try:
                    with open(template_dir / 'project.yml', 'r') as f:
                        config = yaml.safe_load(f)
                        templates[config['name']] = {
                            'config': config,
                            'path': template_dir,
                        }
                except (yaml.YAMLError, KeyError) as e:
                    print(f"Error loading template {template_dir.name}: {e}")

        return templates

    def get_template(self, name: str) -> Optional[Dict[str, Any]]:
        """Get a template by name."""
        return self.available_templates.get(name)

    def list_templates(self) -> List[str]:
        """List all available template names."""
        return list(self.available_templates.keys())

    def create_project(
        self,
        template_name: str,
        project_path: Path,
        context: Dict[str, Any],
        overwrite: bool = False,
    ) -> bool:
        """Create a new project from a template.

        Args:
            template_name: Name of the template to use
            project_path: Path where the project should be created
            context: Dictionary with template variables
            overwrite: Whether to overwrite existing files

        Returns:
            bool: True if project was created successfully

        Raises:
            ValueError: If template is not found
            FileExistsError: If output directory exists and overwrite is False
        """
        template = self.get_template(template_name)
        if not template:
            error_msg = f"Template '{template_name}' not found"
            raise ValueError(error_msg)

        project_path = project_path.absolute()

        # Check if directory exists and handle overwrite
        if project_path.exists():
            if not overwrite:
                error_msg = f"Directory {project_path} already exists. Use --overwrite to ignore."
                raise FileExistsError(error_msg)
        else:
            project_path.mkdir(parents=True, exist_ok=True)

        # Process template files
        config = template['config']
        template_dir = template['path']

        # Create directories first
        for dir_path in config.get('directories', []):
            rendered_dir = self._render_template(dir_path, context)
            full_dir = project_path / rendered_dir
            full_dir.mkdir(parents=True, exist_ok=True)

        # Create files
        for file_spec in config.get('files', []):
            if not isinstance(file_spec, dict):
                continue

            # Handle both 'path' and 'source' keys for file path
            file_path = file_spec.get('path') or file_spec.get('source')
            if not file_path:
                continue

            file_content = file_spec.get('content')
            template_file = file_spec.get('template')

            try:
                # Render the file path
                file_path = self._render_template(file_path, context)
                dest_path = project_path / file_path
                dest_path.parent.mkdir(parents=True, exist_ok=True)

                if file_content is not None:
                    # Use inline content
                    rendered = self._render_template(file_content, context)
                    dest_path.write_text(rendered)
                elif template_file:
                    # Use template file
                    template_path = template_dir / template_file
                    if template_path.exists():
                        template_content = template_path.read_text()
                        rendered = self._render_template(template_content, context)
                        dest_path.write_text(rendered)
                    else:
                        # Create empty file as fallback
                        dest_path.touch()
                else:
                    # Create empty file
                    dest_path.touch()
            except Exception as e:
                print(f"Error processing file {file_path}: {e}")
                raise

        # Run post-create commands
        for cmd in config.get('post_create_commands', []):
            cmd = self._render_template(cmd, context)
            os.system(f"cd {project_path} && {cmd}")

        return True

    def _render_template(self, template_str: str, context: Dict[str, Any]) -> str:
        """Render a template string with the given context."""
        return jinja2.Template(template_str).render(**context)

    def get_template_variables(self, template_name: str) -> Dict[str, Any]:
        """Get the required variables for a template."""
        template = self.get_template(template_name)
        if not template:
            return {}

        # Extract variables from template files
        variables = set()

        # Check config files
        for file_spec in template['config'].get('files', []):
            if isinstance(file_spec, dict):
                content = file_spec.get('content', '')
                template_file = file_spec.get('template')

                if template_file:
                    template_path = template['path'] / template_file
                    if template_path.exists():
                        content = template_path.read_text()

                # Simple variable extraction ({{ variable }})
                import re

                variables.update(re.findall(r'\{\{\s*([^\s}]+)\s*\}\}', content))

        return {var: "" for var in variables}


def get_template_manager() -> TemplateManager:
    """Get a template manager instance."""
    return TemplateManager()



"""Functional tests for the WRD package."""

import os
import shutil
import tempfile
import unittest
from pathlib import Path
from unittest.mock import patch


class TestWRDFunctional(unittest.TestCase):
    """Functional test cases for the WRD package."""

    def setUp(self):
        """Set up test fixtures."""
        self.test_dir = tempfile.mkdtemp()
        self.original_dir = os.getcwd()
        os.chdir(self.test_dir)

        # Create a test project directory
        self.project_name = "test_project"
        self.project_path = Path(self.test_dir) / self.project_name
        self.project_path.mkdir(exist_ok=True)

        # Mock the user's home directory
        self.patcher = patch('pathlib.Path.home')
        self.mock_home = self.patcher.start()
        self.mock_home.return_value = Path(self.test_dir)

    def tearDown(self):
        """Tear down test fixtures."""
        self.patcher.stop()
        os.chdir(self.original_dir)
        shutil.rmtree(self.test_dir, ignore_errors=True)

    def test_project_creation(self):
        """Test creating a new project."""
        from wrd.__main__ import WRDManager

        manager = WRDManager()
        # Store the project object to avoid unused variable warning
        project = manager.create_project(
            name=self.project_name, project_type="python", description="A test project"
        )

        # Check if project directory was created
        self.assertTrue(self.project_path.exists())

        # Check if essential files were created
        self.assertTrue((self.project_path / "README.md").exists())
        self.assertTrue((self.project_path / "CLAUDE.md").exists())
        self.assertTrue((self.project_path / "requirements.txt").exists())

        # Check if .wrd directory was created
        self.assertTrue((self.project_path / ".wrd").exists())
        self.assertTrue((self.project_path / ".wrd" / "config.json").exists())

    @patch('wrd.__main__.subprocess.run')
    def test_commit_command(self, mock_run):
        """Test the commit command."""
        from wrd.__main__ import WRDManager

        # Create a test project
        manager = WRDManager()
        manager.create_project(
            name=self.project_name, project_type="python", description="A test project"
        )

        # Test commit
        manager.commit_project(self.project_name)

        # Check if git commands were called
        self.assertTrue(mock_run.called)

    def test_list_projects(self):
        """Test listing projects."""
        from wrd.__main__ import WRDManager

        # Create a test project
        manager = WRDManager()
        manager.create_project(
            name=self.project_name, project_type="python", description="A test project"
        )

        # Test listing projects
        projects = manager.list_projects()
        self.assertIn(self.project_name, projects)


if __name__ == '__main__':
    unittest.main()



"""Unit tests for the CLI module."""

import os
import sys
from pathlib import Path
from unittest.mock import patch, MagicMock, call

import pytest
from typer.testing import CliRunner

# Add the project root to the Python path
sys.path.insert(0, str(Path(__file__).parent.parent.parent))

from wrd.cli import WRDShell, main as cli_main


class TestCLI:
    """Test cases for the CLI module."""

    @pytest.fixture
    def runner(self):
        """Fixture for CLI runner."""
        return CliRunner()

    @pytest.fixture
    def mock_template_manager(self):
        """Fixture for a mock TemplateManager."""
        with patch('wrd.cli.TemplateManager') as mock:
            yield mock

    def test_cli_help(self, runner):
        """Test the CLI help command."""
        # Since we don't have a direct app object, we'll test the main function
        with patch('sys.argv', ['wrd', '--help']):
            with patch('sys.exit') as mock_exit:
                cli_main()
                # The help should be printed to stdout, but we can't easily capture it here
                # Just verify sys.exit was called (which happens after help is printed)
                mock_exit.assert_called_once()

    def test_version_command(self, capsys):
        """Test the version command."""
        from wrd import __version__

        with patch('sys.argv', ['wrd', '--version']):
            with pytest.raises(SystemExit):
                cli_main()
            captured = capsys.readouterr()
            assert f"WRD version: {__version__}" in captured.out

    @patch(
        'builtins.input', side_effect=['my-project', 'Test User', 'test@example.com']
    )
    @patch('wrd.cli.WRDShell._init_python_project')
    def test_init_command(self, mock_init_project, mock_input, mock_template_manager):
        """Test the init command."""
        # Setup mock
        mock_instance = mock_template_manager.return_value
        mock_instance.list_templates.return_value = ["python-basic"]

        # Create a temporary directory for the test
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            project_path = temp_path / "my-project"

            # Create a WRDShell instance
            shell = WRDShell()

            # Run the init_project method
            shell.init_project(str(project_path))

            # Verify the _init_python_project method was called with the correct arguments
            mock_init_project.assert_called_once()
            args, kwargs = mock_init_project.call_args
            assert str(args[0]) == str(project_path)  # project_path
            assert args[1] == "my-project"  # name
            assert (
                "A new project" in args[2]
            )  # description (default contains this text)

    @patch('wrd.cli.WRDShell')
    def test_shell_command(self, mock_shell_class):
        """Test the shell command."""
        # Setup mock
        mock_shell_instance = MagicMock()
        mock_shell_class.return_value = mock_shell_instance

        # Call the main function with shell command
        with patch('sys.argv', ['wrd', 'shell']):
            cli_main()

        # Verify WRDShell was instantiated and cmdloop was called
        mock_shell_class.assert_called_once()
        mock_shell_instance.cmdloop.assert_called_once()

    def test_list_templates(self, capsys):
        """Test listing templates."""
        # Create a WRDShell instance
        shell = WRDShell()

        # Mock the template manager
        with patch('wrd.cli.TemplateManager') as mock_tm_class:
            mock_tm = mock_tm_class.return_value
            mock_tm.list_templates.return_value = ["python-basic", "nodejs-basic"]

            # Call the method
            shell.do_list_templates("")

            # Capture output
            captured = capsys.readouterr()

            # Verify the output
            assert "Available templates:" in captured.out
            assert "- python-basic" in captured.out
            assert "- nodejs-basic" in captured.out
            mock_tm.list_templates.assert_called_once()


class TestWRDShell:
    """Test cases for the WRDShell class."""

    @pytest.fixture
    def shell(self):
        """Fixture for WRDShell instance."""
        return WRDShell()

    def test_do_help(self, shell, capsys):
        """Test the help command."""
        # Test general help
        shell.do_help("")
        captured = capsys.readouterr()
        assert "Documented commands" in captured.out

        # Test help for specific command
        shell.do_help("init")
        captured = capsys.readouterr()
        assert "Initialize a new project" in captured.out

    def test_do_exit(self, shell):
        """Test the exit command."""
        assert shell.do_exit("") is True

    def test_do_EOF(self, shell):
        """Test the EOF command."""
        assert shell.do_EOF("") is True

    @patch(
        'builtins.input',
        side_effect=['Test Project', 'test@example.com', 'A test project'],
    )
    def test_do_init(self, mock_input, shell, tmp_path):
        """Test the init command."""
        # Create a temporary directory for the test
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            project_path = temp_path / "my-project"

            # Call the init_project method
            shell.init_project(str(project_path))

            # Verify project directory was created
            assert project_path.exists()

            # Verify essential files were created
            assert (project_path / "README.md").exists()
            assert (project_path / "setup.py").exists()
            assert (project_path / "src").exists()
            assert (project_path / "tests").exists()

    @patch('wrd.cli.TemplateManager')
    def test_do_list_templates(self, mock_tm_class, shell, capsys):
        """Test the list-templates command."""
        # Setup mock
        mock_tm = mock_tm_class.return_value
        mock_tm.list_templates.return_value = ["template1", "template2"]

        # Call the method
        shell.do_list_templates("")

        # Verify output
        captured = capsys.readouterr()
        assert "Available templates:" in captured.out
        assert "- template1" in captured.out
        assert "- template2" in captured.out
        mock_tm.list_templates.assert_called_once()

    @patch('wrd.cli.TemplateManager')
    def test_do_use_template(self, mock_tm_class, shell, tmp_path, capsys):
        """Test the use-template command."""
        # Setup mock
        mock_tm = mock_tm_class.return_value
        mock_tm.list_templates.return_value = ["python-basic"]
        mock_tm.create_project.return_value = 0

        # Call the method
        shell.do_use_template("python-basic my-project")

        # Verify template manager was called correctly
        mock_tm.create_project.assert_called_once()
        call_args = mock_tm.create_project.call_args[1]
        assert call_args["template_name"] == "python-basic"
        assert call_args["context"]["project_name"] == "my-project"

        # Verify success message
        captured = capsys.readouterr()
        assert "Project created successfully" in captured.out

    def test_complete_use_template(self, shell):
        """Test tab completion for the use-template command."""
        # Mock the template manager
        with patch('wrd.cli.TemplateManager') as mock_tm_class:
            mock_tm = mock_tm_class.return_value
            mock_tm.list_templates.return_value = ["python-basic", "nodejs-basic"]

            # Test template name completion
            completions = shell.complete_use_template("", "use-template ".split(), 1)
            assert set(completions) == {"python-basic", "nodejs-basic"}

            # Test no completion after template name
            assert (
                shell.complete_use_template(
                    "python-basic ", "use-template python-basic ".split(), 2
                )
                == []
            )



"""Version information for WRD package."""

# This version should match the version in pyproject.toml
__version__ = "0.1.2"
__author__ = "Tom Sapletta"
__license__ = "Apache-2.0"
__email__ = "info@softreck.dev"



"""WRD Command Line Interface."""

import os
import sys
import yaml
from pathlib import Path
from typing import Dict, Any, Optional, List
from rich.console import Console
from rich.prompt import Prompt, Confirm, IntPrompt
from rich.panel import Panel
from rich.table import Table
from rich.progress import track
import subprocess
import shutil

console = Console()


class WRDConfig:
    """Manage WRD configuration."""

    def __init__(self):
        self.config_dir = Path.home() / ".wrd"
        self.config_file = self.config_dir / "config.yaml"
        self.templates_dir = self.config_dir / "templates"
        self.ensure_directories()
        self.config = self.load_config()

    def ensure_directories(self):
        """Ensure configuration directories exist."""
        self.config_dir.mkdir(exist_ok=True)
        self.templates_dir.mkdir(exist_ok=True)

    def load_config(self) -> Dict[str, Any]:
        """Load configuration from YAML file."""
        if not self.config_file.exists():
            return {
                "projects_dir": str(Path.home() / "projects"),
                "editor": os.environ.get("EDITOR", "code"),
                "default_language": "python",
                "templates": {},
            }

        with open(self.config_file, 'r') as f:
            return yaml.safe_load(f) or {}

    def save_config(self):
        """Save configuration to YAML file."""
        with open(self.config_file, 'w') as f:
            yaml.dump(self.config, f, default_flow_style=False)


class WRDShell:
    """Interactive WRD shell."""

    def __init__(self):
        self.config = WRDConfig()
        self.current_project = None
        self.console = Console()

    def print_banner(self):
        """Print WRD banner."""
        banner = """
        ██╗    ██╗██████╗ ██████╗ 
        ██║    ██║██╔══██╗██╔══██╗
        ██║ █╗ ██║██████╔╝██║  ██║
        ██║███╗██║██╔══██╗██║  ██║
        ╚███╔███╔╝██║  ██║██████╔╝
         ╚══╝╚══╝ ╚═╝  ╚═╝╚═════╝ 
        WRD - WRonai Development Tool
        """
        self.console.print(Panel.fit(banner, style="bold blue"))

    def get_menu_choice(
        self, title: str, options: List[Dict[str, str]], timeout: int = 10
    ) -> str:
        """Display a menu and get user choice."""
        table = Table(title=title, show_header=False, show_lines=True)
        table.add_column("Option", style="cyan")
        table.add_column("Description")

        for i, option in enumerate(options, 1):
            table.add_row(str(i), option['description'])

        self.console.print(table)

        try:
            choice = IntPrompt.ask(
                "\nEnter your choice",
                choices=[str(i) for i in range(1, len(options) + 1)],
                timeout=timeout,
            )
            return options[int(choice) - 1]['command']
        except Exception:
            self.console.print("\n[bold red]Timeout reached. Using default option.[/]")
            return options[0]['command']

    def init_project(self, project_path: Optional[str] = None):
        """Initialize a new project."""
        if project_path is None:
            project_path = Prompt.ask("Enter project path", default=str(Path.cwd()))

        project_path = Path(project_path).absolute()

        if project_path.exists() and any(project_path.iterdir()):
            self.console.print(f"[yellow]Directory {project_path} is not empty.[/]")
            if not Confirm.ask("Do you want to initialize a project here anyway?"):
                return

        # Get project details
        project_name = project_path.name
        language = Prompt.ask(
            "Project language",
            default=self.config.config.get('default_language', 'python'),
        )
        description = Prompt.ask("Project description", default="")

        # Create project structure
        project_path.mkdir(parents=True, exist_ok=True)

        # Initialize project based on language
        if language == 'python':
            self._init_python_project(project_path, project_name, description)
        # Add more language support here

        self.console.print(
            f"[green]✓ Project {project_name} initialized successfully![/]"
        )

    def _init_python_project(self, project_path: Path, name: str, description: str):
        """Initialize a Python project."""
        # Create basic Python project structure
        (project_path / 'src' / name.replace('-', '_')).mkdir(
            parents=True, exist_ok=True
        )
        (project_path / 'tests').mkdir(exist_ok=True)

        # Create pyproject.toml
        pyproject = f"""[build-system]
requires = ["setuptools>=42.0"]
build-backend = "setuptools.build_meta"

[project]
name = "{name}"
version = "0.1.0"
description = "{description}"
authors = [
    {{ name = "Your Name", email = "your.email@example.com" }}
]
readme = "README.md"
requires-python = ">=3.8"

[project.optional-dependencies]
dev = [
    "pytest>=6.0",
    "black>=21.12b0",
    "isort>=5.10.1",
    "mypy>=0.910",
    "pytest-cov>=2.0"
]
"""
        (project_path / 'pyproject.toml').write_text(pyproject)

        # Create README.md
        readme = f"""# {name}

{description}

## Installation

```bash
pip install -e .
```

## Development

Install development dependencies:

```bash
pip install -e ".[dev]"
```

Run tests:

```bash
pytest
```
"""
        (project_path / 'README.md').write_text(readme)

        # Create .gitignore
        gitignore = """# Python
__pycache__/
*.py[cod]
*$py.class
*.so
.Python
build/
develop-eggs/
dist/
downloads/
eggs/
.eggs/
lib/
lib64/
parts/
sdist/
var/
wheels/
*.egg-info/
.installed.cfg
*.egg

# Virtual Environment
venv/
env/

# IDE
.idea/
.vscode/
*.swp
*.swo

# Testing
.coverage
htmlcov/
.pytest_cache/
"""
        (project_path / '.gitignore').write_text(gitignore)

        # Initialize git repository
        try:
            subprocess.run(['git', 'init'], cwd=project_path, check=True)
            subprocess.run(['git', 'add', '.'], cwd=project_path, check=True)
            subprocess.run(
                ['git', 'commit', '-m', 'Initial commit'], cwd=project_path, check=True
            )
        except Exception as e:
            self.console.print(
                f"[yellow]Warning: Failed to initialize git repository: {e}[/]"
            )

    def shell(self):
        """Start interactive shell."""
        self.print_banner()

        while True:
            try:
                options = [
                    {'command': 'init', 'description': 'Initialize a new project'},
                    {'command': 'list', 'description': 'List all projects'},
                    {'command': 'config', 'description': 'Configure WRD'},
                    {'command': 'exit', 'description': 'Exit WRD shell'},
                ]

                choice = self.get_menu_choice("WRD - Main Menu", options, timeout=60)

                if choice == 'init':
                    self.init_project()
                elif choice == 'list':
                    self.list_projects()
                elif choice == 'config':
                    self.configure()
                elif choice == 'exit':
                    self.console.print("[green]Goodbye![/]")
                    break

            except KeyboardInterrupt:
                self.console.print("\n[red]Operation cancelled.[/]")
            except Exception as e:
                self.console.print(f"[red]Error: {e}[/]")

    def list_projects(self):
        """List all projects in the projects directory."""
        projects_dir = Path(
            self.config.config.get('projects_dir', Path.home() / 'projects')
        )

        if not projects_dir.exists():
            self.console.print("[yellow]No projects directory found.[/]")
            return

        table = Table(title="Projects")
        table.add_column("Name", style="cyan")
        table.add_column("Path")
        table.add_column("Modified")

        for project_dir in projects_dir.iterdir():
            if project_dir.is_dir():
                modified = datetime.fromtimestamp(project_dir.stat().st_mtime).strftime(
                    '%Y-%m-%d %H:%M'
                )
                table.add_row(project_dir.name, str(project_dir), modified)

        self.console.print(table)

    def configure(self):
        """Configure WRD settings."""
        self.console.print("[bold]Current Configuration:[/]")
        for key, value in self.config.config.items():
            self.console.print(f"  [cyan]{key}:[/] {value}")

        if Confirm.ask("\nDo you want to change any settings?"):
            setting = Prompt.ask("Enter setting name")
            if setting in self.config.config:
                if isinstance(self.config.config[setting], bool):
                    new_value = Confirm.ask(f"New value for {setting}")
                else:
                    new_value = Prompt.ask(f"New value for {setting}")
                self.config.config[setting] = new_value
                self.config.save_config()
                self.console.print("[green]✓ Configuration updated.[/]")
            else:
                self.console.print("[red]Invalid setting name.[/]")


def main():
    """Entry point for the WRD CLI."""
    if len(sys.argv) > 1 and sys.argv[1] == 'shell':
        WRDShell().shell()
    elif len(sys.argv) > 1 and sys.argv[1] == 'init':
        WRDShell().init_project(sys.argv[2] if len(sys.argv) > 2 else None)
    else:
        console = Console()
        console.print("WRD - WRonai Development Tool")
        console.print("\nUsage:")
        console.print("  wrd shell     - Start interactive shell")
        console.print("  wrd init [path] - Initialize a new project")
        console.print("  wrd --help    - Show this help")


if __name__ == "__main__":
    main()



"""Basic tests for the WRD package."""

import os
import sys
import unittest
from unittest.mock import patch

# Add the project root to the Python path
sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))


class TestWRD(unittest.TestCase):
    """Test cases for the WRD package."""

    def setUp(self):
        """Set up test fixtures."""
        pass

    def test_import(self):
        """Test that the package can be imported."""
        import wrd

        self.assertIsNotNone(wrd)
        self.assertTrue(hasattr(wrd, '__version__'))

    @patch('wrd.__main__.main')
    def test_main_module(self, mock_main):
        """Test that the __main__ module can be executed."""
        import wrd.__main__

        wrd.__main__.main()
        mock_main.assert_called_once()

    def test_cli_commands(self):
        """Test that the basic CLI commands are available."""
        from wrd.__main__ import main

        # This is a simple test that the main function exists
        self.assertTrue(callable(main))


if __name__ == '__main__':
    unittest.main()



"""Integration tests for the WRD CLI."""

import os
import sys
import tempfile
import shutil
from pathlib import Path
from typing import Dict, Any
from unittest.mock import patch

import pytest

# Add the project root to the Python path
sys.path.insert(0, str(Path(__file__).parent.parent.parent))

from wrd.cli import WRDShell, main as cli_main
from wrd.template_manager import TemplateManager


class TestCLIIntegration:
    """Integration tests for the WRD CLI."""

    @pytest.fixture
    def shell(self):
        """Fixture for WRDShell instance."""
        return WRDShell()

    def test_init_command_integration(self, shell, tmp_path):
        """Test the init command with a real template manager."""
        # Create a temporary directory for the test
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            project_path = temp_path / "my-project"

            # Mock user input
            with patch(
                'builtins.input',
                side_effect=['Test Project', 'test@example.com', 'A test project'],
            ):
                # Call the init_project method
                shell.init_project(str(project_path))

            # Verify project structure
            assert project_path.exists()
            assert (project_path / "README.md").exists()
            assert (project_path / "setup.py").exists()
            assert (project_path / "src").exists()
            assert (project_path / "tests").exists()

    @patch(
        'builtins.input',
        side_effect=['Test Project', 'test@example.com', 'A test project'],
    )
    def test_use_template_command(self, mock_input, shell, tmp_path, capsys):
        """Test the use-template command with a real template manager."""
        # Create a temporary directory for the test
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = Path(temp_dir)
            project_path = temp_path / "test-project"

            # Call the use_template method
            shell.use_template(str(project_path), "python-basic")

            # Verify project structure
            assert project_path.exists()
            assert (project_path / "README.md").exists()
            assert (project_path / "setup.py").exists()
            assert (project_path / "src").exists()
            assert (project_path / "tests").exists()

    def test_list_templates_integration(self, shell, capsys):
        """Test the list-templates command with a real template manager."""
        # Call the list_templates method
        shell.list_templates()

        # Verify output
        captured = capsys.readouterr()
        assert "Available templates:" in captured.out
        assert "python-basic" in captured.out


class TestTemplateManagerIntegration:
    """Integration tests for the TemplateManager class."""

    def test_create_project_integration(self, tmp_path):
        """Test creating a project from a template."""
        # Setup
        manager = TemplateManager()
        project_path = tmp_path / "test-project"

        # Get the first available template
        templates = manager.list_templates()
        assert templates, "No templates found for testing"
        template_name = templates[0]

        # Create project
        context = {
            "project_name": "test-project",
            "author": "Test User",
            "email": "test@example.com",
            "description": "A test project",
            "license": "MIT",
            "version": "0.1.0",
        }

        result = manager.create_project(
            template_name=template_name,
            project_path=project_path,
            context=context,
            overwrite=False,
        )

        # Verify
        assert result is True
        assert project_path.exists()
        assert (project_path / "README.md").exists()

    def test_get_template_integration(self):
        """Test getting a template by name."""
        manager = TemplateManager()

        # Get the first available template
        templates = manager.list_templates()
        assert templates, "No templates found for testing"
        template_name = templates[0]

        # Get template
        template = manager.get_template(template_name)
        assert template is not None
        assert "config" in template
        assert "path" in template
        assert template["config"]["name"] == template_name



"""WRD (WRonai Development) - A powerful workflow automation tool for developers."""

import sys
from typing import TYPE_CHECKING, Any

# Import version info
from .__version__ import __author__, __email__, __version__

# Lazy imports to prevent circular imports during build
if not TYPE_CHECKING and not sys.version_info >= (3, 7):
    from importlib import import_module

    def __getattr__(name: str) -> Any:
        if name in {"WRDShell", "TemplateManager", "get_template_manager"}:
            import importlib

            module = importlib.import_module(
                f"wrd.{name.lower()}"
                if name != 'get_template_manager'
                else 'wrd.template_manager'
            )
            return getattr(module, name)
        raise AttributeError(f"module 'wrd' has no attribute '{name}'")

else:
    try:
        from .cli import WRDShell
        from .template_manager import TemplateManager, get_template_manager
    except ImportError:
        # This can happen during build time
        WRDShell = None  # type: ignore
        TemplateManager = None  # type: ignore
        get_template_manager = None  # type: ignore

__all__ = [
    '__version__',
    '__author__',
    '__email__',
    'WRDShell',
    'TemplateManager',
    'get_template_manager',
]

__license__ = "Apache-2.0"
